cda.section = {}

local createEntry

-- Accepts a <structuredBody> element as argument
-- DisplayName is optional
-- TemplateIds is optiona.  If provided,
-- it should be a list of tables with the following
-- optional entries:
-- root, extension, assigningAuthorityName
function cda.section.new(SB, Code, Title, TemplateIds, DisplayName)
   local Comp = SB:append(xml.ELEMENT, 'component')
   local S = Comp:append(xml.ELEMENT, 'section')
   if TemplateIds then
      cda.util.addTemplateIds(S, TemplateIds)
   end
   local C = S:append(xml.ELEMENT, 'code')
   C:setAttr('code', Code)
   cda.codesystem.set(C, cda.cs.LOINC)
   if DisplayName then
      C:setAttr('displayName', DisplayName)
   end
   S:append(xml.ELEMENT, 'title'):setText(Title)
   return S
end

function cda.section.setId(S, Id)
   for i=1,#S do
      if S[i]:nodeType() == 'element' and S[i]:nodeName() == 'code' then
         local I = S:insert(i, xml.ELEMENT, 'id')
         I:setAttr('root', Id)
         return I
      end
   end
   error('Section has no <code> element in front of which id should be inserted.', 2)
end

-- Accepts a <section> element.
-- Returns the <text> element.  The inner text will be set to InnerText,
-- if specified.
function cda.section.addText(S, InnerText)
   local T = S:append(xml.ELEMENT, 'text')
   if InnerText then
      S:child('text'):append(xml.TEXT, InnerText)
   end
   return T
end

-- NegationInd is an optional boolean parameter.
-- By default, NegationInd will be omitted.
function cda.section.addObservation(S, ClassCode, MoodCode, NegationInd)
   local O = cda.entry.new(S, 'observation', ClassCode, MoodCode)
   if NegationInd ~= nil then
      O:setAttr('negationInd', tostring(NegationInd))
   end
   return O
end

-- EntryTypeCode is optional
function cda.section.addActEntry(S, ClassCode, MoodCode, EntryTypeCode)
   return cda.entry.new(S, 'act', ClassCode, MoodCode, EntryTypeCode)
end

-- EntryTypeCode is optional
function cda.section.addProcedureEntry(S, ClassCode, MoodCode, EntryTypeCode)
   return cda.entry.new(S, 'procedure', ClassCode, MoodCode, EntryTypeCode)
end

-- EntryTypeCode is optional
function cda.section.addEncounterEntry(S, ClassCode, MoodCode, EntryTypeCode)
   return cda.entry.new(S, 'encounter', ClassCode, MoodCode, EntryTypeCode)
end

-- EntryTypeCode is optional
function cda.section.addSubstanceAdmin(S, ClassCode, MoodCode, EntryTypeCode)
   return cda.entry.new(S, 'substanceAdministration', ClassCode, MoodCode, EntryTypeCode)
end

